bagging <-
function(formula, data, mfinal=100, control) {

formula<- as.formula(formula)
vardep <- data[,as.character(formula[[2]])]
        n <- length(data[,1])
nclases <- nlevels(vardep)

#pesos <- rep(1/n,n)
#data<-data.frame(data,pesos)#creo que en bagging no hace falta
     arboles <- list() #Creamos una lista para guardar los arboles
replicas <- array(0, c(n,mfinal))

for (m in 1:mfinal) {

boostrap<- sample(1:n,replace=TRUE)
fit <- rpart(formula,data=data[boostrap,], control=control)
        flearn <- predict(fit,newdata=data,type="class")
ind<-as.numeric(vardep != flearn) #crear un vector indicador
err<- sum(ind)/n                       #calcula el error en esa iteraci<f3>n

arboles[[m]] <- fit#Guardamos los arboles
replicas[,m]<-boostrap


}

pred<- data.frame(rep(0,n)) 
# Crea un dataframe para guardar las pred, al inicio esta vacio, pero luego se va agnadiendo

	#2012-05-16 nueva medida de importancia
	#sustituye a acum
	nvar<-dim(varImp(arboles[[1]], surrogates = FALSE, competes = FALSE))[1]
	imp<- array(0, c(mfinal,nvar))  #Creo una matriz para guardar el "improve" de cada variable conforme evoluciona boosting



for (m in 1:mfinal) {
if(m==1){pred <- predict(arboles[[m]],data,type="class")} 
else{pred <- data.frame(pred,predict(arboles[[m]],data,type="class"))} 

		k <- varImp(arboles[[m]], surrogates = FALSE, competes = FALSE)
		imp[m,] <-k[sort(row.names(k)), ]

}

classfinal <- array(0, c(n,nlevels(vardep)))
for (i in 1:nlevels(vardep)){
 classfinal[,i] <- matrix(as.numeric(pred==levels(vardep)[i]),nrow=n)%*%rep(1,mfinal)
}

predclass <- rep("O",n)
for(i in 1:n){
predclass[i] <- as.character(levels(vardep)[(order(classfinal[i,],decreasing=TRUE)[1])])
}

#normalizar la importancia de las variables, las ponderaciones son todas iguales en bagging

	pond<-rep(1,mfinal)
	imppond<-as.vector(as.vector(pond)%*%imp)
	imppond<-imppond/sum(imppond)*100
	names(imppond)<-sort(row.names(k))


#Para que devuelva las probabilidades a posteriori
classfinal/apply(classfinal,1,sum)->votosporc


ans<- list(formula=formula,trees=arboles,votes=classfinal,prob=votosporc,class=predclass, samples=replicas, importance=imppond)

class(ans) <- "bagging"
ans

}
